/***************************************************************************
 *   Copyright (C) 2007 by David Sansome <davidsansome@gmail.com>          *
 *                         Andrea Grandi <a.grandi@gmail.com>              *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "mainwindow.h"
#include "facebookservice.h"
#include "flickrservice.h"
#include "gpicasaservice.h"

#include <QtopiaApplication>
#include <QMenu>
#include <QSoftMenuBar>
#include <QImageDocumentSelectorDialog>
#include <QThumbnail>
#include <QSettings>
#include <QKeyEvent>

MainWindow::MainWindow(QWidget* parent, Qt::WindowFlags f)
	: QWidget(parent, f),
	  m_service(NULL),
	  m_createAlbumDialog(NULL)
{
	m_ui.setupUi(this);
	
	m_settings = new QSettings(this);
	loadAccounts();
	if (m_services.count() > 0)
		m_ui.accountList->setCurrentRow(0);
	
	QMenu* accountsMenu = QSoftMenuBar::menuFor(m_ui.accountList);
	accountsMenu->addAction(QPixmap(":/data/facebook.png"), "Add Facebook...", this, SLOT(addFacebook()));
	accountsMenu->addAction(QPixmap(":/data/flickr.png"), "Add Flickr...", this, SLOT(addFlickr()));
#ifdef HAS_GDATA
	accountsMenu->addAction("Add Picasa...", this, SLOT(addPicasa()));
#endif
	accountsMenu->addSeparator();
	accountsMenu->addAction(QPixmap(":icon/cancel.svg"), "Remove account", this, SLOT(removeAccount()));

	QMenu* imagesMenu = QSoftMenuBar::menuFor(m_ui.photoList);
	imagesMenu->addAction(QPixmap(":icon/new.svg"), "Add photo...", this, SLOT(addPhoto()));
	imagesMenu->addAction(QPixmap(":icon/cancel.svg"), "Remove photo", this, SLOT(removePhoto()));
	
	QMenu* albumsMenu = QSoftMenuBar::menuFor(m_ui.albumList);
	albumsMenu->addAction(QPixmap(":icon/new.svg"), "Create new album...", this, SLOT(createAlbum()));
	
	installEventFilter(this);
	m_ui.accountList->installEventFilter(this);
	m_ui.photoList->installEventFilter(this);
	m_ui.albumList->installEventFilter(this);
	m_ui.tagPhotoList->installEventFilter(this);
	
	connect(m_ui.accountList, SIGNAL(itemActivated(QListWidgetItem*)), SLOT(accountSelected()));
	connect(m_ui.photoList, SIGNAL(itemActivated(QListWidgetItem*)), SLOT(photoSelected()));
	connect(m_ui.albumList, SIGNAL(itemActivated(QListWidgetItem*)), SLOT(albumSelected()));
	connect(m_ui.tagPhotoList, SIGNAL(itemActivated(QListWidgetItem*)), SLOT(tagPhoto(QListWidgetItem*)));
	
	connect(m_ui.morePhotosButton, SIGNAL(clicked()), SLOT(morePhotosButtonClicked()));
	connect(m_ui.tagButton, SIGNAL(clicked()), SLOT(tagPhotos()));
}

MainWindow::~MainWindow()
{
	saveAccounts();
	
	foreach (Album* a, m_albums)
		delete a;
	foreach (Service* s, m_services)
		delete s;
}

void MainWindow::saveAccounts()
{
	m_settings->beginWriteArray("accounts", m_services.count());
	int i=0;
	foreach (Service* service, m_services)
	{
		m_settings->setArrayIndex(i++);
		m_settings->setValue("type", service->type());
		service->save(m_settings);
	}
	m_settings->endArray();
}

void MainWindow::loadAccounts()
{
	foreach (Service* s, m_services)
		delete s;
	m_services.clear();
	
	int size = m_settings->beginReadArray("accounts");
	for (int i=0 ; i<size ; ++i)
	{
		m_settings->setArrayIndex(i);
		
		QString type = m_settings->value("type").toString();

		Service* service = NULL;
		QPixmap pixmap;
		
		if (type == "FacebookService")
		{
			service = new FacebookService(m_settings);
			pixmap = QPixmap(":/data/facebook.png");
		}
		else if (type == "FlickrService")
		{
			service = new FlickrService(m_settings);
			pixmap = QPixmap(":/data/flickr.png");
		}
#ifdef HAS_GDATA
		else if (type == "GPicasaService")
			service = new GPicasaService(m_settings);
#endif
		
		if (service != NULL)
		{
			m_services.append(service);
			new QListWidgetItem(pixmap, service->name(), m_ui.accountList);
		}
	}
	m_settings->endArray();
}

void MainWindow::addFacebook()
{
	addService(new FacebookService(), QPixmap(":/data/facebook.png"));
}

void MainWindow::addFlickr()
{
	addService(new FlickrService(), QPixmap(":/data/flickr.png"));
}

void MainWindow::addPicasa()
{
#ifdef HAS_GDATA
	addService(new GPicasaService());
#else
	QMessageBox::information(this, "Google Picasa", "Qt 4.3 is required for Google Picasa Web support");
#endif
}

void MainWindow::addService(Service* service, const QPixmap& pixmap)
{
	setProgress(m_ui.accountPage, "Connecting...");
	
	bool success = service->login();
	
	m_ui.stackedWidget->setCurrentWidget(m_ui.accountPage);
	
	if (!success)
	{
		delete service;
		return;
	}
	
	connect(service, SIGNAL(uploadProgress(int, int)), SLOT(uploadProgress(int, int))); 

	m_services.append(service);
	new QListWidgetItem(pixmap, service->name(), m_ui.accountList);
	saveAccounts();
}

void MainWindow::accountSelected()
{
	m_service = m_services[m_ui.accountList->currentRow()];

	m_ui.tagButton->setVisible(m_service->supportsTagging());
	
	m_ui.stackedWidget->setCurrentWidget(m_ui.photoPage);
	if (m_photos.count() == 0)
		addPhoto();
}

void MainWindow::removeAccount()
{
	int row = m_ui.accountList->currentRow();
	
	delete m_services[row];
	m_services.removeAt(row);
	m_ui.accountList->takeItem(row);
	
	saveAccounts();
}

void MainWindow::addPhoto()
{
	QImageDocumentSelectorDialog* dialog = new QImageDocumentSelectorDialog(this);
	
	if (QtopiaApplication::execDialog(dialog) == QDialog::Accepted)
	{
		QContent content(dialog->selectedDocument());
		QIODevice* device = content.open();
		QByteArray data = device->readAll();
		QImage image(QImage::fromData(data));
		
		m_photos.append(Photo(image, content.name()));
		
		new QListWidgetItem(QIcon(QPixmap::fromImage(image)), content.name(), m_ui.photoList);
		
		delete device;
	}

	delete dialog;
}

void MainWindow::removePhoto()
{
	int row = m_ui.photoList->currentRow();
	
	m_photos.removeAt(row);
	m_ui.photoList->takeItem(row);
}

bool MainWindow::eventFilter(QObject*, QEvent* e)
{
	if (e->type() != QEvent::KeyPress)
		return false;
	
	QKeyEvent* event = (QKeyEvent*) e;
	if (event->key() != Qt::Key_Back)
		return false;
	
	if (m_ui.stackedWidget->currentWidget() == m_ui.accountPage || m_ui.stackedWidget->currentWidget() == m_ui.donePage)
		QCoreApplication::exit();
	else if (m_ui.stackedWidget->currentWidget() == m_ui.progressPage)
	{
		m_service->abort();
		m_ui.stackedWidget->setCurrentIndex(m_abortPage);
	}
	else
		m_ui.stackedWidget->setCurrentIndex(m_ui.stackedWidget->currentIndex() - 1);
	
	return true;
}

void MainWindow::setProgress(QWidget* abortPage, const QString& line1, const QString& line2, int maximum)
{
	m_abortPage = m_ui.stackedWidget->indexOf(abortPage);
	
	m_ui.progressLine1->setText("<b>" + line1 + "</b>");
	m_ui.progressLine2->setText(line2);
	m_ui.progressLine2->setVisible(!line2.isEmpty());
	m_ui.progressBar->setMinimum(0);
	m_ui.progressBar->setMaximum(maximum);
	m_ui.progressBar->setValue(0);
	m_ui.stackedWidget->setCurrentWidget(m_ui.progressPage);
}

void MainWindow::photoSelected()
{
	setProgress(m_ui.photoPage, "Getting albums...");
	
	foreach (Album* a, m_albums)
		delete a;
	m_albums.clear();
	m_ui.albumList->clear();
	
	m_albums = m_service->getAlbums();
	
	new QListWidgetItem("(Default album)", m_ui.albumList);
	foreach (Album* album, m_albums)
		new QListWidgetItem(album->name() + " (" + QString::number(album->count()) + ")", m_ui.albumList);
	
	m_ui.stackedWidget->setCurrentWidget(m_ui.albumPage);
	m_ui.albumList->setCurrentRow(m_ui.albumList->currentRow());
}

void MainWindow::createAlbum()
{
	if (!m_createAlbumDialog)
	{
		m_createAlbumDialog = new QDialog(this);
		m_createAlbumDialogUi.setupUi(m_createAlbumDialog);
	}
	
	m_createAlbumDialogUi.albumName->setText("");
	
	if (QtopiaApplication::execDialog(m_createAlbumDialog) != QDialog::Accepted)
		return;
	if (m_createAlbumDialogUi.albumName->text().isEmpty())
		return;
	
	setProgress(m_ui.albumPage, "Creating album...", m_createAlbumDialogUi.albumName->text());
	
	Album* album = m_service->createAlbum(m_createAlbumDialogUi.albumName->text());
	m_albums.append(album);
	new QListWidgetItem(album->name() + " (" + QString::number(album->count()) + ")", m_ui.albumList);
	
	m_ui.stackedWidget->setCurrentWidget(m_ui.albumPage);
}

void MainWindow::albumSelected()
{
	int row = m_ui.albumList->currentRow();
	if (row <= 0)
		m_album = NULL;
	else
		m_album = m_albums[row-1];
	
	setProgress(m_ui.albumPage, "Uploading photos...", QString::null, m_photos.count());
	
	int i = 0;
	bool error = false;
	QList<Photo>::iterator it; // Have to use an iterator so we can modify id
	for (it=m_photos.begin() ; it!=m_photos.end() ; ++it)
	{
		QString fileName((*it).fileName);
		
		m_ui.progressLine2->setText(fileName);
		m_ui.progressLine2->show();
		
		QString lower(fileName.toLower());
		if (!lower.endsWith(".jpg") && !lower.endsWith(".jpeg"))
			fileName += ".jpg";
		
		(*it).id = m_service->uploadPhoto(m_album, (*it).image, fileName);
		m_ui.progressBar->setValue(++i);
		
		if (m_service->requestAborted() || m_service->isError())
		{
			error = true;
			break;
		}
	}
	
	if (!error)
	{
		m_ui.doneText->setText("Your photos have been uploaded.  You may have to visit the " + m_service->serviceName() + " website to approve the photos before they become publicly visible.");
		m_ui.stackedWidget->setCurrentWidget(m_ui.donePage);
	}
}

void MainWindow::morePhotosButtonClicked()
{
	m_photos.clear();
	m_ui.photoList->clear();
	m_ui.stackedWidget->setCurrentWidget(m_ui.accountPage);
}

void MainWindow::uploadProgress(int bytes, int totalBytes)
{
	m_ui.progressBar->setMaximum(totalBytes);
	m_ui.progressBar->setValue(bytes);
}

void MainWindow::tagPhotos()
{
	setProgress(m_ui.donePage, "Getting friends list...");
	m_service->prepareForTagging();
	m_ui.stackedWidget->setCurrentWidget(m_ui.tagPage);

	m_ui.tagPhotoList->clear();
	foreach (Photo photo, m_photos)
	{
		QListWidgetItem* item = new QListWidgetItem(m_ui.tagPhotoList);
		item->setText(photo.fileName);
		item->setIcon(QIcon(QPixmap::fromImage(photo.image)));
		item->setData(QVariant::UserType, photo.id);
	}
}

void MainWindow::tagPhoto(QListWidgetItem* item)
{
	qulonglong id = item->data(QVariant::UserType).toULongLong();
	
	// This is a bit nasty
	QImage image;
	foreach (Photo photo, m_photos)
		if (photo.id == id)
			image = photo.image;
	if (image.isNull())
		return;

	setProgress(m_ui.tagPage, "Tagging photo...");
	m_service->tagPhoto(id, image);
	m_ui.stackedWidget->setCurrentWidget(m_ui.tagPage);
}


